'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var isEqual = require('fast-deep-equal/react');
var react = require('react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var isEqual__default = /*#__PURE__*/_interopDefaultLegacy(isEqual);

function useDeepCompareCallback(fn, deps) {
    const signal = react.useRef(0);
    const prevDeps = react.useRef(deps);
    if (!isEqual__default["default"](prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return react.useCallback(fn, [signal.current]);
}

function useDeepCompareEffect(fn, deps) {
    const signal = react.useRef(0);
    const prevDeps = react.useRef(deps);
    if (!isEqual__default["default"](prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    react.useEffect(fn, [signal.current]);
}

function useDeepCompareMemo(fn, deps) {
    const signal = react.useRef(0);
    const prevDeps = react.useRef(deps);
    if (!isEqual__default["default"](prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return react.useMemo(fn, [signal.current]);
}

function useInterval(callback, delay) {
    const callbackRef = react.useRef(callback);
    react.useEffect(() => {
        callbackRef.current = callback;
    }, [callback]);
    react.useEffect(() => {
        const tick = () => {
            if (callbackRef.current === undefined)
                return;
            callbackRef.current();
        };
        const id = setInterval(tick, delay);
        return () => clearInterval(id);
    }, [delay]);
}

function usePreviousState(value) {
    const ref = react.useRef(value);
    react.useEffect(() => {
        ref.current = value;
    }, [value]);
    return ref.current;
}

exports.useDeepCompareCallback = useDeepCompareCallback;
exports.useDeepCompareEffect = useDeepCompareEffect;
exports.useDeepCompareMemo = useDeepCompareMemo;
exports.useInterval = useInterval;
exports.usePreviousState = usePreviousState;
