import isEqual from 'fast-deep-equal/react';
import { useRef, useCallback, useEffect, useMemo } from 'react';

function useDeepCompareCallback(fn, deps) {
    const signal = useRef(0);
    const prevDeps = useRef(deps);
    if (!isEqual(prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return useCallback(fn, [signal.current]);
}

function useDeepCompareEffect(fn, deps) {
    const signal = useRef(0);
    const prevDeps = useRef(deps);
    if (!isEqual(prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    useEffect(fn, [signal.current]);
}

function useDeepCompareMemo(fn, deps) {
    const signal = useRef(0);
    const prevDeps = useRef(deps);
    if (!isEqual(prevDeps.current, deps)) {
        signal.current += 1;
    }
    prevDeps.current = deps;
    return useMemo(fn, [signal.current]);
}

function useInterval(callback, delay) {
    const callbackRef = useRef(callback);
    useEffect(() => {
        callbackRef.current = callback;
    }, [callback]);
    useEffect(() => {
        const tick = () => {
            if (callbackRef.current === undefined)
                return;
            callbackRef.current();
        };
        const id = setInterval(tick, delay);
        return () => clearInterval(id);
    }, [delay]);
}

function usePreviousState(value) {
    const ref = useRef(value);
    useEffect(() => {
        ref.current = value;
    }, [value]);
    return ref.current;
}

export { useDeepCompareCallback, useDeepCompareEffect, useDeepCompareMemo, useInterval, usePreviousState };
